// Chuck Benz, Hollis, NH   Copyright (c)2002
//
// The information and description contained herein is the
// property of Chuck Benz.
//
// Permission is granted for any reuse of this information
// and description as long as this copyright notice is
// preserved.  Modifications may be made as long as this
// notice is preserved.

// per Widmer and Franaszek

`timescale 1ns / 1ps

module encode (datain, dispin, dataout, dispout) ;
  input [8:0]   datain ;
  input 	dispin ;  // 0 = neg disp; 1 = pos disp
  output [9:0]	dataout ;
  output	dispout ;


  wire wai = datain[0] ;
  wire wbi = datain[1] ;
  wire wci = datain[2] ;
  wire wdi = datain[3] ;
  wire wei = datain[4] ;
  wire wfi = datain[5] ;
  wire wgi = datain[6] ;
  wire whi = datain[7] ;
  wire wki = datain[8] ;

  wire aeqb = (wai & wbi) | (!wai & !wbi) ;
  wire ceqd = (wci & wdi) | (!wci & !wdi) ;
  wire l22 = (wai & wbi & !wci & !wdi) |
	     (wci & wdi & !wai & !wbi) |
	     ( !aeqb & !ceqd) ;
  wire l40 = wai & wbi & wci & wdi ;
  wire l04 = !wai & !wbi & !wci & !wdi ;
  wire l13 = ( !aeqb & !wci & !wdi) |
	     ( !ceqd & !wai & !wbi) ;
  wire l31 = ( !aeqb & wci & wdi) |
	     ( !ceqd & wai & wbi) ;

  // The 5B/6B encoding

  wire wao = wai ;
  wire wbo = (wbi & !l40) | l04 ;
  wire wco = l04 | wci | (wei & wdi & !wci & !wbi & !wai) ;
  wire wdo = wdi & ! (wai & wbi & wci) ;
  wire weo = (wei | l13) & ! (wei & wdi & !wci & !wbi & !wai) ;
  wire wio = (l22 & !wei) |
	    (wei & !wdi & !wci & !(wai&wbi)) |  // D16, D17, D18
	    (wei & l40) |
	    (wki & wei & wdi & wci & !wbi & !wai) | // K.28
	    (wei & !wdi & wci & !wbi & !wai) ;

  // pds16 indicates cases where d-1 is assumed + to get our encoded value
  wire pd1s6 = (wei & wdi & !wci & !wbi & !wai) | (!wei & !l22 & !l31) ;
  // nds16 indicates cases where d-1 is assumed - to get our encoded value
  wire nd1s6 = wki | (wei & !l22 & !l13) | (!wei & !wdi & wci & wbi & wai) ;

  // ndos6 is pds16 cases where d-1 is + yields - disp out - all of them
  wire ndos6 = pd1s6 ;
  // pdos6 is nds16 cases where d-1 is - yields + disp out - all but one
  wire pdos6 = wki | (wei & !l22 & !l13) ;


  // some Dx.7 and all Kx.7 cases result in run length of 5 case unless
  // an alternate coding is used (referred to as Dx.A7, normal is Dx.P7)
  // specifically, D11, D13, D14, D17, D18, D19.
  wire alt7 = wfi & wgi & whi & (wki |
			      (dispin ? (!wei & wdi & l31) : (wei & !wdi & l13))) ;


  wire wfo = wfi & ! alt7 ;
  wire wgo = wgi | (!wfi & !wgi & !whi) ;
  wire who = whi ;
  wire wjo = (!whi & (wgi ^ wfi)) | alt7 ;

  // nd1s4 is cases where d-1 is assumed - to get our encoded value
  wire nd1s4 = wfi & wgi ;
  // pd1s4 is cases where d-1 is assumed + to get our encoded value
  wire pd1s4 = (!wfi & !wgi) | (wki & ((wfi & !wgi) | (!wfi & wgi))) ;

  // ndos4 is pd1s4 cases where d-1 is + yields - disp out - just some
  wire ndos4 = (!wfi & !wgi) ;
  // pdos4 is nd1s4 cases where d-1 is - yields + disp out
  wire pdos4 = wfi & wgi & whi ;

  // only legal K codes are K28.0->.7, K23/27/29/30.7
  //	K28.0->7 is ei=di=ci=1,bi=ai=0
  //	K23 is 10111
  //	K27 is 11011
  //	K29 is 11101
  //	K30 is 11110 - so K23/27/29/30 are ei & l31
  wire illegalk = wki &
		  (wai | wbi | !wci | !wdi | !wei) & // not K28.0->7
		  (!wfi | !wgi | !whi | !wei | !l31) ; // not K23/27/29/30.7

  // now determine whether to do the complementing
  // complement if prev disp is - and pd1s6 is set, or + and nd1s6 is set
  wire compls6 = (pd1s6 & !dispin) | (nd1s6 & dispin) ;

  // disparity out of 5b6b is disp in with pdso6 and ndso6
  // pds16 indicates cases where d-1 is assumed + to get our encoded value
  // ndos6 is cases where d-1 is + yields - disp out
  // nds16 indicates cases where d-1 is assumed - to get our encoded value
  // pdos6 is cases where d-1 is - yields + disp out
  // disp toggles in all ndis16 cases, and all but that 1 nds16 case

  wire disp6 = dispin ^ (ndos6 | pdos6) ;

  wire compls4 = (pd1s4 & !disp6) | (nd1s4 & disp6) ;
  assign dispout = disp6 ^ (ndos4 | pdos4) ;

  assign dataout = {(wjo ^ compls4), (who ^ compls4),
		    (wgo ^ compls4), (wfo ^ compls4),
		    (wio ^ compls6), (weo ^ compls6),
		    (wdo ^ compls6), (wco ^ compls6),
		    (wbo ^ compls6), (wao ^ compls6)} ;

endmodule
